****************************************************************************
****************************************************************************
**                                                               
**  Monster - variable handling for TI-92
**
**  by Patrick Davidson (eeulplek@hotmail.com, http://www.ocf.berkeley.edu/~pad)
**                  
**  Last updated April 23, 2019
**
****************************************************************************
****************************************************************************

******************************************** locate level used in saved game

Load_Last_Level:
        pea     main_folder(pc)
        move.w  FOLDER_LIST_HANDLE,-(sp)
        JSR_ROM FindSymEntry            ; find main folder
        lea     6(sp),sp                ; A0 -> SYM_ENTRY of main folder
        move.w  tios::SYM_ENTRY.hVal(a0),d0
        tios::DEREF     d0,a4           ; A6 -> main folder
        
        addq.l  #2,a4
        move.w  (a4)+,d6                ; D6 = remaining entries in folder
        
saved_search_loop:
        move.l  (a4),d0
        cmp.l   level_name(a5),d0
        bne.s   \skip
        move.l  4(a4),d0
        cmp.l   level_name+4(a5),d0
        bne.s   \skip        
        
        move.w  tios::SYM_ENTRY.hVal(a4),d0
        tios::DEREF     d0,a0
        lea     33(a0),a1
        move.l  a1,level_pointer(a5)        
        bsr     verify_level
        cmp.w   num_levels(a5),d0
        beq.s   \found

\skip:  lea     12(a4),a4               ; advance to next SYM_ENTRY in folder
        subq.w  #1,d6
        bne.s   saved_search_loop
        
        moveq   #0,d0
        rts

\found: moveq   #1,d0
        rts

******************************************** validate file in A0
*
* Takes A0 as a pointer to the variable data.
* Returns zero flag set for valid, clear for invalid.
* Returns number of levels in file in D0 on success.
* Modifies A0/D0-D2.
*
********

verify_level:
        move.w  (a0),d2                 ; D2 = file size
        
        addq.l  #3,a0
        move.b  (a0)+,d0                ; read level count
        lsl.w   #8,d0
        move.b  (a0)+,d0
        
        cmp.b   #$6a,(a0)+              ; compare signature
        bne.s   variable_not_level
        cmp.b   #$4f,(a0)+
        bne.s   variable_not_level
        lea     10(a0),a0               ; skip level name
        tst.b   (a0)+                   ; check required version (must be 0)
        bne.s   variable_not_level
        tst.b   (a0)+
        bne.s   variable_not_level
        lea     14(a0),a0               ; skip flags
        
        move.w  d0,d1                   ; verify expected size
        mulu    #216,d1
        add.w   #33,d1
        cmp.w   d1,d2
variable_not_level:        
        rts

******************************************** create level list

Select_Level:
        moveq   #0,d7                   ; D7 = # of levels found so far
        lea     map(a5),a6              ; A0 -> variable data structures
        
        pea     main_folder(pc)
        move.w  FOLDER_LIST_HANDLE,-(sp)
        JSR_ROM FindSymEntry            ; find main folder
        lea     6(sp),sp                ; A0 -> SYM_ENTRY of main folder
        move.w  tios::SYM_ENTRY.hVal(a0),d0
        tios::DEREF     d0,a4           ; A6 -> main folder
        
        addq.l  #2,a4
        move.w  (a4)+,d6                ; D6 = remaining entries in folder
        
var_search_loop:
        move.w  tios::SYM_ENTRY.hVal(a4),d0
        tios::DEREF     d0,a0
        bsr     verify_level
        bne.s   skip_variable

        move.l  a4,(a6)+                ; save SYM_ENTRY pointer to level
        move.w  d0,(a6)+                ; save level count
        addq.w  #1,d7                   ; increment count of levels found

skip_variable:
        lea     12(a4),a4               ; advance to next SYM_ENTRY in folder
        
        cmp.w   #12,d7                  ; maximum number to display
        beq.s   var_search_done
        subq.w  #1,d6
        bne.s   var_search_loop

var_search_done:
        subq.w  #1,d7
        bge.s   levels_found
        move.w  #EXIT_NOLEVEL,exit_status(a5)
        moveq   #0,d0
        rts

******************************************** display level list

levels_found:
        move.w  d7,d6                   ; D6 = # left to display
        beq     level_selected    

        moveq   #1,d0
        bsr     Set_Font
        lea     (a5),a0                 ; clear screen
        move.w  #PLANE_SIZE/4-1,d0
\c:     clr.l   (a0)+
        dbra    d0,\c

        clr.w   temp_buffer+8(a5)

        moveq   #0,d5                   ; D5 = Y coordinate
        lea     map(a5),a4              ; A4 = position in list
level_list_loop:
        move.l  (a4)+,a3
        move.l  (a3)+,temp_buffer(a5)   ; copy level name
        move.l  (a3)+,temp_buffer+4(a5)
        move.w  (a4)+,-(sp)             ; push level count
        pea     temp_buffer(a5)         ; push pointer to level name
        pea     level_list_format(pc)
        pea     _string_buffer(a5)
        JSR_ROM sprintf
        lea     14(sp),sp

        lea     _string_buffer(a5),a0
        moveq   #0,d1
        move.w  d5,d0
        addq.w  #8,d5
        bsr     Display_String

        dbra    d6,level_list_loop

******************************************** select level from list

        moveq   #0,d6                   ; D6 = level selected
level_select_loop:
        bsr     Wait_VBL
        bsr     Invert_Piece
        bsr     Display_Half
        bsr     Invert_Piece
        bsr     Read_All_Keys

        GETEDGE 0,0,0,5
        bne.s   \noup
        tst.w   d6
        beq.s   \noup
        subq.w  #1,d6
\noup:

        GETEDGE 0,2,0,7
        bne.s   \nodown
        cmp.w   d6,d7
        beq.s   \nodown
        addq.w  #1,d6
\nodown:

    IFND ti89
        GETEDGE THIS_IS_FOR,TI92_ONLY,6,6
        beq.s   level_selected
    ENDIF
        GETEDGE 1,0,9,1                 ; ENTER    
        beq.s   level_selected
        GETEDGE 0,4,0,3                 ; 2nd
        beq.s   level_selected

        GETEDGE 6,0,8,6                 ; ESC
        beq.s   level_load_failed
        GETKEY  3,0,9,6                 ; Test status of . key
        bne.s   level_select_loop

level_load_failed:
        moveq   #0,d0
        rts

level_selected:
        mulu    #6,d6
        lea     map(a5),a0
        lea     0(a0,d6.w),a1           ; A1 = entry in level list
        move.l  (a1)+,a3                ; A3 -> SYM_ENTRY of the level
        move.l  (a3)+,level_name(a5)
        move.l  (a3)+,level_name+4(a5)
        move.w  2(a3),d0

Load_D0_Level:
        tios::DEREF     d0,a0
        lea     33(a0),a1
        move.l  a1,level_pointer(a5)
        bsr     verify_level
        move.w  d0,num_levels(a5)
        rts

Invert_Piece:
        lea     (a5),a0
        move.w  d6,d0
        mulu    #8*WIDTH_BYTES,d0       
        moveq   #WIDTH_BYTES*2-1,d1
        add.w   d0,a0
\l:     not.l   (a0)+
        dbra    d1,\l
        rts
        
******************************************** data

main_folder:
        dc.b    "main",0
        
level_list_format:
        dc.b    '%s (%d)',0
        
        EVEN
